@@ -1,25 +1,13 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
-
-   Without limiting anything contained in the foregoing, this file,
-   which is part of C Driver for MySQL (Connector/C), is also subject to the
-   Universal FOSS Exception, version 1.0, a copy of which can be found at
-   http://oss.oracle.com/licenses/universal-foss-exception.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -98,7 +86,7 @@ C_MODE_END
 static const char *args_separator= "----args-separator----";
 inline static void set_args_separator(char** arg)
 {
-  assert(my_getopt_use_args_separator);
+  DBUG_ASSERT(my_getopt_use_args_separator);
   *arg= (char*)args_separator;
 }
 
@@ -286,7 +274,7 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
                                       (char **) &my_login_path, found_no_defaults);
 
     if (! my_defaults_group_suffix)
-      my_defaults_group_suffix= getenv("MYSQL_GROUP_SUFFIX");
+      my_defaults_group_suffix= getenv(STRINGIFY_ARG(DEFAULT_GROUP_SUFFIX_ENV));
 
     if (forced_extra_defaults && !defaults_already_read)
     {
@@ -600,8 +588,6 @@ int load_defaults(const char *conf_file, const char **groups,
   return my_load_defaults(conf_file, groups, argc, argv, &default_directories);
 }
 
-/** A global to turn off or on reading the mylogin file. On by default */
-my_bool my_defaults_read_login_file= TRUE;
 /*
   Read options from configurations files
 
@@ -686,18 +672,16 @@ int my_load_defaults(const char *conf_file, const char **groups,
     DBUG_RETURN(error);
   }
 
-  if (my_defaults_read_login_file)
-  {
-    /* Read options from login group. */
-    if (my_default_get_login_file(my_login_file, sizeof(my_login_file)) &&
-      (error= my_search_option_files(my_login_file, argc, argv, &args_used,
+  /* Read options from login group. */
+  if (my_default_get_login_file(my_login_file, sizeof(my_login_file)) &&
+      (error= my_search_option_files(my_login_file,argc, argv, &args_used,
                                      handle_default_option, (void *) &ctx,
                                      dirs, true, found_no_defaults)))
-    {
-      free_root(&alloc, MYF(0));
-      DBUG_RETURN(error);
-    }
+  {
+    free_root(&alloc,MYF(0));
+    DBUG_RETURN(error);
   }
+
   /*
     Here error contains <> 0 only if we have a fully specified conf_file
     or a forced default file
@@ -1043,12 +1027,8 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
       goto err;
     }
 
-    /* comments are not supported in login file */
-    if (!is_login_file)
-      end = remove_end_comment(ptr);
-    else
-      end = ptr + strlen(ptr);
 
+    end= remove_end_comment(ptr);
     if ((value= strchr(ptr, '=')))
       end= value;				/* Option without argument */
     for ( ; my_isspace(&my_charset_latin1, end[-1]) ; end--)
@@ -1328,7 +1308,7 @@ static int add_directory(MEM_ROOT *alloc, const char *dir, const char **dirs)
     return 1;  /* Failure */
   /* Should never fail if DEFAULT_DIRS_SIZE is correct size */
   err= array_append_string_unique(p, dirs, DEFAULT_DIRS_SIZE);
-  assert(err == FALSE);
+  DBUG_ASSERT(err == FALSE);
 
   return 0;
 }
