@@ -16,6 +16,7 @@
 #include <my_global.h>
 #include <mysql/plugin_keyring.h>
 #include "keyring.h"
+#include "buffered_file_io.h"
 
 #ifdef _WIN32
 #define MYSQL_DEFAULT_KEYRINGFILE MYSQL_KEYRINGDIR"\\keyring"
@@ -54,14 +55,13 @@ my_bool create_keyring_dir_if_does_not_exist(const char *keyring_file_path)
   return FALSE;
 }
 
-int check_keyring_file_data(MYSQL_THD thd  __attribute__((unused)),
-                            struct st_mysql_sys_var *var  __attribute__((unused)),
+int check_keyring_file_data(MYSQL_THD thd  MY_ATTRIBUTE((unused)),
+                            struct st_mysql_sys_var *var  MY_ATTRIBUTE((unused)),
                             void *save, st_mysql_value *value)
 {
   char            buff[FN_REFLEN+1];
   const char      *keyring_filename;
   int             len = sizeof(buff);
-  Buffered_file_io keyring_io(logger.get());
   boost::movelib::unique_ptr<IKeys_container> new_keys(new Keys_container(logger.get()));
 
   (*(const char **) save)= NULL;
@@ -76,7 +76,8 @@ int check_keyring_file_data(MYSQL_THD thd  __attribute__((unused)),
   }
   try
   {
-    if (new_keys->init(&keyring_io, keyring_filename))
+    IKeyring_io *keyring_io(new Buffered_file_io(logger.get()));
+    if (new_keys->init(keyring_io, keyring_filename))
     {
       mysql_rwlock_unlock(&LOCK_keyring);
       return 1;
@@ -128,9 +129,9 @@ static int keyring_init(MYSQL_PLUGIN plugin_info)
         "directory gets provided");
       return FALSE;
     }
-    Buffered_file_io keyring_io(logger.get());
     keys.reset(new Keys_container(logger.get()));
-    if (keys->init(&keyring_io, keyring_file_data_value))
+    IKeyring_io *keyring_io= new Buffered_file_io(logger.get());
+    if (keys->init(keyring_io, keyring_file_data_value))
     {
       is_keys_container_initialized = FALSE;
       logger->log(MY_ERROR_LEVEL, "keyring_file initialization failure. Please check"
@@ -152,7 +153,7 @@ static int keyring_init(MYSQL_PLUGIN plugin_info)
   }
 }
 
-int keyring_deinit(void *arg __attribute__((unused)))
+int keyring_deinit(void *arg MY_ATTRIBUTE((unused)))
 {
   //not taking a lock here as the calls to keyring_deinit are serialized by
   //the plugin framework
@@ -166,20 +167,18 @@ int keyring_deinit(void *arg __attribute__((unused)))
 my_bool mysql_key_fetch(const char *key_id, char **key_type, const char *user_id,
                         void **key, size_t *key_len)
 {
-  return mysql_key_fetch<Buffered_file_io, Key>(key_id, key_type, user_id, key,
-                                                key_len);
+  return mysql_key_fetch<Key>(key_id, key_type, user_id, key, key_len);
 }
 
 my_bool mysql_key_store(const char *key_id, const char *key_type,
                         const char *user_id, const void *key, size_t key_len)
 {
-  return mysql_key_store<Buffered_file_io, Key>(key_id, key_type, user_id, key,
-                                                key_len);
+  return mysql_key_store<Key>(key_id, key_type, user_id, key, key_len);
 }
 
 my_bool mysql_key_remove(const char *key_id, const char *user_id)
 {
-  return mysql_key_remove<Buffered_file_io, Key>(key_id, user_id);
+  return mysql_key_remove<Key>(key_id, user_id);
 }
 
 
@@ -188,7 +187,6 @@ my_bool mysql_key_generate(const char *key_id, const char *key_type,
 {
   try
   {
-    Buffered_file_io keyring_io(logger.get());
     boost::movelib::unique_ptr<IKey> key_candidate(new Key(key_id, key_type, user_id, NULL, 0));
 
     boost::movelib::unique_ptr<uchar[]> key(new uchar[key_len]);
