@@ -1,21 +1,15 @@
 /*
- * Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2016 Oracle and/or its affiliates. All rights reserved.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License, version 2.0,
- * as published by the Free Software Foundation.
- *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an additional
- * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License, version 2.0, for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
@@ -92,17 +86,25 @@ Error Connection::connect_to_localhost(const std::string &named_pipe_or_unix_soc
   strncpy(addr.sun_path, named_pipe_or_unix_socket.c_str(), sizeof(addr.sun_path)-1);
   addr.sun_path[sizeof(addr.sun_path)-1] = 0;
 
-  return connect((sockaddr*)&addr, sizeof(addr));
+  return connect(&addr, sizeof(addr));
 #else
   return Error(CR_SOCKET_CREATE_ERROR, "Named pipes aren't supported on current OS");
 #endif // defined(HAVE_SYS_UN_H)
 }
 
-Error Connection::connect(sockaddr *addr, const std::size_t addr_size)
+Error Connection::connect(sockaddr_un *addr, const std::size_t addr_size)
 {
-  my_socket s = ::socket(addr->sa_family,
-                         SOCK_STREAM,
-                         addr->sa_family == AF_UNIX ? 0: IPPROTO_TCP);
+#if defined(HAVE_SYS_UN_H)
+  my_socket s = socket(AF_UNIX, SOCK_STREAM, 0);
+  return connect(s, (sockaddr*)addr, addr_size);
+#else
+  return Error(CR_SOCKET_CREATE_ERROR, "Unix socket aren't supported on current OS");
+#endif // defined(HAVE_SYS_UN_H)
+}
+
+Error Connection::connect(sockaddr_in *addr, const std::size_t addr_size)
+{
+  my_socket s = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
 
   return connect(s, (sockaddr*)addr, addr_size);
 }
