@@ -1,21 +1,15 @@
 /*
- * Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2016 Oracle and/or its affiliates. All rights reserved.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License, version 2.0,
- * as published by the Free Software Foundation.
- *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an additional
- * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License, version 2.0, for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
@@ -34,14 +28,17 @@
 #endif
 
 #include "ngs_common/protocol_protobuf.h"
+#include <boost/scoped_ptr.hpp>
 #include "mysqlx_protocol.h"
+#include "mysqlx_crud.h"
 #include "mysqlx_resultset.h"
 #include "mysqlx_row.h"
 #include "mysqlx_error.h"
 #include "mysqlx_version.h"
 
 #include "my_config.h"
-#include "ngs_common/bind.h"
+
+#include <boost/bind.hpp>
 
 #ifdef MYSQLXTEST_STANDALONE
 #include "mysqlx/auth_mysql41.h"
@@ -171,16 +168,12 @@ static void throw_server_error(const Mysqlx::Error &error)
 }
 
 
-XProtocol::XProtocol(const Ssl_config &ssl_config,
-                     const std::size_t timeout,
-                     const bool dont_wait_for_disconnect,
-                     const Internet_protocol ip_mode)
+XProtocol::XProtocol(const Ssl_config &ssl_config, const std::size_t timeout, const bool dont_wait_for_disconnect)
 : m_sync_connection(ssl_config.key, ssl_config.ca, ssl_config.ca_path,
                     ssl_config.cert, ssl_config.cipher, ssl_config.tls_version, timeout),
   m_client_id(0),
   m_trace_packets(false), m_closed(true),
-  m_dont_wait_for_disconnect(dont_wait_for_disconnect),
-  m_ip_mode(ip_mode)
+  m_dont_wait_for_disconnect(dont_wait_for_disconnect)
 {
   if (getenv("MYSQLX_TRACE_CONNECTION"))
     m_trace_packets = true;
@@ -234,12 +227,7 @@ void XProtocol::connect(const std::string &host, int port)
   memset(&hints, 0, sizeof(hints));
   hints.ai_socktype= SOCK_STREAM;
   hints.ai_protocol= IPPROTO_TCP;
-  hints.ai_family= AF_UNSPEC;
-
-  if (IPv6 == m_ip_mode)
-    hints.ai_family = AF_INET6;
-  else if (IPv4 == m_ip_mode)
-    hints.ai_family = AF_INET;
+  hints.ai_family= AF_INET;
 
   gai_errno= getaddrinfo(host.c_str(), port_buf, &hints, &res_lst);
   if (gai_errno != 0)
@@ -247,7 +235,7 @@ void XProtocol::connect(const std::string &host, int port)
 
   for (t_res= res_lst; t_res; t_res= t_res->ai_next)
   {
-    error = m_sync_connection.connect((sockaddr*)t_res->ai_addr, t_res->ai_addrlen);
+    error = m_sync_connection.connect((sockaddr_in*)t_res->ai_addr, t_res->ai_addrlen);
 
     if (!error)
       break;
@@ -295,7 +283,7 @@ void XProtocol::fetch_capabilities()
 {
   send(Mysqlx::Connection::CapabilitiesGet());
   int mid;
-  ngs::unique_ptr<Message> message(recv_raw(mid));
+  boost::scoped_ptr<Message> message(recv_raw(mid));
   if (mid != Mysqlx::ServerMessages::CONN_CAPABILITIES)
     throw Error(CR_COMMANDS_OUT_OF_SYNC, "Unexpected response received from server");
   m_capabilities = *static_cast<Mysqlx::Connection::Capabilities*>(message.get());
@@ -334,7 +322,7 @@ void XProtocol::close()
     int mid;
     try
     {
-      ngs::unique_ptr<Message> message(recv_raw(mid));
+      boost::scoped_ptr<Message> message(recv_raw(mid));
       if (mid != Mysqlx::ServerMessages::OK)
         throw Error(CR_COMMANDS_OUT_OF_SYNC, "Unexpected message received in response to Session.Close");
 
@@ -348,13 +336,6 @@ void XProtocol::close()
   }
 }
 
-unsigned long XProtocol::get_received_msg_counter(const std::string &id) const
-{
-  std::map<std::string, unsigned long>::const_iterator i =
-      m_received_msg_counters.find(id);
-  return i == m_received_msg_counters.end() ? 0ul : i->second;
-}
-
 void XProtocol::perform_close()
 {
   if (m_dont_wait_for_disconnect)
@@ -364,7 +345,7 @@ void XProtocol::perform_close()
   }
 
   int mid;
-  ngs::unique_ptr<Message> message(recv_raw(mid));
+  boost::scoped_ptr<Message> message(recv_raw(mid));
   std::stringstream s;
 
   s << "Unexpected message received with id:" << mid << " while waiting for disconnection";
@@ -372,19 +353,19 @@ void XProtocol::perform_close()
   throw Error(CR_COMMANDS_OUT_OF_SYNC, s.str());
 }
 
-ngs::shared_ptr<Result> XProtocol::recv_result()
+boost::shared_ptr<Result> XProtocol::recv_result()
 {
   return new_result(true);
 }
 
-ngs::shared_ptr<Result> XProtocol::new_empty_result()
+boost::shared_ptr<Result> XProtocol::new_empty_result()
 {
-  ngs::shared_ptr<Result> empty_result(new Result(shared_from_this(), false, false));
+  boost::shared_ptr<Result> empty_result(new Result(shared_from_this(), false, false));
 
   return empty_result;
 }
 
-ngs::shared_ptr<Result> XProtocol::execute_sql(const std::string &sql)
+boost::shared_ptr<Result> XProtocol::execute_sql(const std::string &sql)
 {
   {
     Mysqlx::Sql::StmtExecute exec;
@@ -396,7 +377,7 @@ ngs::shared_ptr<Result> XProtocol::execute_sql(const std::string &sql)
   return new_result(true);
 }
 
-ngs::shared_ptr<Result> XProtocol::execute_stmt(const std::string &ns, const std::string &sql, const std::vector<ArgumentValue> &args)
+boost::shared_ptr<Result> XProtocol::execute_stmt(const std::string &ns, const std::string &sql, const std::vector<ArgumentValue> &args)
 {
   {
     Mysqlx::Sql::StmtExecute exec;
@@ -450,28 +431,28 @@ ngs::shared_ptr<Result> XProtocol::execute_stmt(const std::string &ns, const std
   return new_result(true);
 }
 
-ngs::shared_ptr<Result> XProtocol::execute_find(const Mysqlx::Crud::Find &m)
+boost::shared_ptr<Result> XProtocol::execute_find(const Mysqlx::Crud::Find &m)
 {
   send(m);
 
   return new_result(true);
 }
 
-ngs::shared_ptr<Result> XProtocol::execute_update(const Mysqlx::Crud::Update &m)
+boost::shared_ptr<Result> XProtocol::execute_update(const Mysqlx::Crud::Update &m)
 {
   send(m);
 
   return new_result(false);
 }
 
-ngs::shared_ptr<Result> XProtocol::execute_insert(const Mysqlx::Crud::Insert &m)
+boost::shared_ptr<Result> XProtocol::execute_insert(const Mysqlx::Crud::Insert &m)
 {
   send(m);
 
   return new_result(false);
 }
 
-ngs::shared_ptr<Result> XProtocol::execute_delete(const Mysqlx::Crud::Delete &m)
+boost::shared_ptr<Result> XProtocol::execute_delete(const Mysqlx::Crud::Delete &m)
 {
   send(m);
 
@@ -494,7 +475,7 @@ void XProtocol::setup_capability(const std::string &name, const bool value)
     m_last_result->buffer();
 
   int mid;
-  ngs::unique_ptr<Message> msg(recv_raw(mid));
+  boost::scoped_ptr<Message> msg(recv_raw(mid));
 
   if (Mysqlx::ServerMessages::ERROR == mid)
     throw_server_error(*(Mysqlx::Error*)msg.get());
@@ -522,7 +503,7 @@ void XProtocol::authenticate_mysql41(const std::string &user, const std::string
 
   {
     int mid;
-    ngs::unique_ptr<Message> message(recv_raw(mid));
+    boost::scoped_ptr<Message> message(recv_raw(mid));
     switch (mid)
     {
       case Mysqlx::ServerMessages::SESS_AUTHENTICATE_CONTINUE:
@@ -563,7 +544,6 @@ void XProtocol::authenticate_mysql41(const std::string &user, const std::string
 
       case Mysqlx::ServerMessages::ERROR:
         throw_server_error(*static_cast<Mysqlx::Error*>(message.get()));
-        break;
 
       default:
         throw Error(CR_MALFORMED_PACKET, "Unexpected message received from server during authentication");
@@ -575,7 +555,7 @@ void XProtocol::authenticate_mysql41(const std::string &user, const std::string
   while (!done)
   {
     int mid;
-    ngs::unique_ptr<Message> message(recv_raw(mid));
+    boost::scoped_ptr<Message> message(recv_raw(mid));
     switch (mid)
     {
       case Mysqlx::ServerMessages::SESS_AUTHENTICATE_OK:
@@ -584,7 +564,6 @@ void XProtocol::authenticate_mysql41(const std::string &user, const std::string
 
       case Mysqlx::ServerMessages::ERROR:
         throw_server_error(*static_cast<Mysqlx::Error*>(message.get()));
-        break;
 
       case Mysqlx::ServerMessages::NOTICE:
         dispatch_notice(static_cast<Mysqlx::Notice::Frame*>(message.get()));
@@ -617,7 +596,7 @@ void XProtocol::authenticate_plain(const std::string &user, const std::string &p
   while (!done)
   {
     int mid;
-    ngs::unique_ptr<Message> message(recv_raw(mid));
+    boost::scoped_ptr<Message> message(recv_raw(mid));
     switch (mid)
     {
       case Mysqlx::ServerMessages::SESS_AUTHENTICATE_OK:
@@ -626,7 +605,6 @@ void XProtocol::authenticate_plain(const std::string &user, const std::string &p
 
       case Mysqlx::ServerMessages::ERROR:
         throw_server_error(*static_cast<Mysqlx::Error*>(message.get()));
-        break;
 
       case Mysqlx::ServerMessages::NOTICE:
         dispatch_notice(static_cast<Mysqlx::Notice::Frame*>(message.get()));
@@ -842,12 +820,10 @@ Message *XProtocol::recv_payload(const int mid, const std::size_t msglen)
 
     if (!ret_val->IsInitialized())
     {
-      std::string err("Message is not properly initialized: ");
-      err += ret_val->InitializationErrorString();
-
       delete[] mbuf;
       delete ret_val;
-
+      std::string err("Message is not properly initialized: ");
+      err += ret_val->InitializationErrorString();
       throw Error(CR_MALFORMED_PACKET, err);
     }
   }
@@ -858,7 +834,7 @@ Message *XProtocol::recv_payload(const int mid, const std::size_t msglen)
   }
 
   delete[] mbuf;
-  update_received_msg_counter(ret_val);
+
   return ret_val;
 }
 
@@ -915,7 +891,7 @@ void XProtocol::throw_mysqlx_error(const Error &error)
   throw error;
 }
 
-ngs::shared_ptr<Result> XProtocol::new_result(bool expect_data)
+boost::shared_ptr<Result> XProtocol::new_result(bool expect_data)
 {
   if (m_last_result)
     m_last_result->buffer();
@@ -925,25 +901,6 @@ ngs::shared_ptr<Result> XProtocol::new_result(bool expect_data)
   return m_last_result;
 }
 
-void XProtocol::update_received_msg_counter(const Message* msg)
-{
-  const std::string &id = msg->GetDescriptor()->full_name();
-  ++m_received_msg_counters[id];
-
-  if (id != Mysqlx::Notice::Frame::descriptor()->full_name()) return;
-
-  static const std::string *notice_type_id[] = {
-      &Mysqlx::Notice::Warning::descriptor()->full_name(),
-      &Mysqlx::Notice::SessionVariableChanged::descriptor()->full_name(),
-      &Mysqlx::Notice::SessionStateChanged::descriptor()->full_name()};
-  static const unsigned notice_type_id_size =
-      sizeof(notice_type_id) / sizeof(notice_type_id[0]);
-  const ::google::protobuf::uint32 notice_type =
-      static_cast<const Mysqlx::Notice::Frame *>(msg)->type() - 1u;
-  if (notice_type < notice_type_id_size)
-    ++m_received_msg_counters[*notice_type_id[notice_type]];
-}
-
 #ifdef WIN32
 #  pragma pop_macro("ERROR")
 #endif
