@@ -1,21 +1,15 @@
 /*
- * Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License, version 2.0,
- * as published by the Free Software Foundation.
- *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an additional
- * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License, version 2.0, for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
@@ -36,6 +30,7 @@
 
 #include "ngs_common/xdatetime.h"
 #include "mysqlx_protocol.h"
+#include "mysqlx_common.h"
 
 
 namespace Mysqlx
@@ -66,7 +61,7 @@ namespace mysqlx
     DECIMAL
   };
 
-  struct ColumnMetadata
+  struct MYSQLXTEST_PUBLIC ColumnMetadata
   {
     FieldType type;
     std::string name;
@@ -88,7 +83,7 @@ namespace mysqlx
     uint32_t content_type;
   };
 
-  class Row
+  class MYSQLXTEST_PUBLIC Row
   {
   public:
     ~Row();
@@ -114,36 +109,36 @@ namespace mysqlx
 
   private:
     friend class Result;
-    Row(ngs::shared_ptr<std::vector<ColumnMetadata> > columns, Mysqlx::Resultset::Row *data);
+    Row(boost::shared_ptr<std::vector<ColumnMetadata> > columns, Mysqlx::Resultset::Row *data);
 
     void check_field(int field, FieldType type) const;
 
-    ngs::shared_ptr<std::vector<ColumnMetadata> > m_columns;
+    boost::shared_ptr<std::vector<ColumnMetadata> > m_columns;
     Mysqlx::Resultset::Row *m_data;
   };
 
-  class ResultData
+  class MYSQLXTEST_PUBLIC ResultData
   {
   public:
-    ResultData(ngs::shared_ptr<std::vector<ColumnMetadata> > columns);
-    ngs::shared_ptr<std::vector<ColumnMetadata> > columnMetadata(){ return m_columns; }
-    void add_row(ngs::shared_ptr<Row> row);
+    ResultData(boost::shared_ptr<std::vector<ColumnMetadata> > columns);
+    boost::shared_ptr<std::vector<ColumnMetadata> > columnMetadata(){ return m_columns; }
+    void add_row(boost::shared_ptr<Row> row);
     void rewind();
     void tell(size_t &record);
     void seek(size_t record);
-    ngs::shared_ptr<Row> next();
+    boost::shared_ptr<Row> next();
   private:
-    ngs::shared_ptr<std::vector<ColumnMetadata> > m_columns;
-    std::vector<ngs::shared_ptr<Row> > m_rows;
+    boost::shared_ptr<std::vector<ColumnMetadata> > m_columns;
+    std::vector<boost::shared_ptr<Row> > m_rows;
     size_t m_row_index;
   };
 
-  class Result
+  class MYSQLXTEST_PUBLIC Result
   {
   public:
     ~Result();
 
-    ngs::shared_ptr<std::vector<ColumnMetadata> > columnMetadata();
+    boost::shared_ptr<std::vector<ColumnMetadata> > columnMetadata();
     int64_t lastInsertId() const { return m_last_insert_id; }
     std::string lastDocumentId();
     const std::vector<std::string>& lastDocumentIds();
@@ -153,7 +148,7 @@ namespace mysqlx
     bool ready();
     void wait();
 
-    ngs::shared_ptr<Row> next();
+    boost::shared_ptr<Row> next();
     bool nextDataSet();
     void flush();
 
@@ -179,10 +174,10 @@ namespace mysqlx
   private:
     Result();
     Result(const Result &o);
-    Result(ngs::shared_ptr<XProtocol>owner, bool expect_data, bool expect_ok = true);
+    Result(boost::shared_ptr<XProtocol>owner, bool expect_data, bool expect_ok = true);
 
     void read_metadata();
-    ngs::shared_ptr<Row> read_row();
+    boost::shared_ptr<Row> read_row();
     void read_stmt_ok();
 
     bool handle_notice(int32_t type, const std::string &data);
@@ -194,8 +189,8 @@ namespace mysqlx
     int              current_message_id;
 
     friend class XProtocol;
-    ngs::weak_ptr<XProtocol>m_owner;
-    ngs::shared_ptr<std::vector<ColumnMetadata> > m_columns;
+    boost::weak_ptr<XProtocol>m_owner;
+    boost::shared_ptr<std::vector<ColumnMetadata> > m_columns;
     int64_t m_last_insert_id;
     std::vector<std::string> m_last_document_ids;
     int64_t m_affected_rows;
@@ -203,8 +198,8 @@ namespace mysqlx
 
     std::vector<Warning> m_warnings;
 
-    std::vector<ngs::shared_ptr<ResultData> > m_result_cache;
-    ngs::shared_ptr<ResultData> m_current_result;
+    std::vector<boost::shared_ptr<ResultData> > m_result_cache;
+    boost::shared_ptr<ResultData> m_current_result;
     size_t m_result_index;
 
     enum {
