@@ -1,21 +1,14 @@
 /*
-   Copyright (c) 2013, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2013, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -27,7 +20,6 @@
 #include "violite.h"                    // Vio
 #include "channel_info.h"               // Channel_info
 #include "connection_handler_manager.h" // Connection_handler_manager
-#include "init_net_server_extension.h"
 #include "mysqld.h"                     // key_socket_tcpip
 #include "log.h"                        // sql_print_error
 #include "sql_class.h"                  // THD
@@ -41,13 +33,7 @@
 #include <sys/un.h>
 #endif
 #ifdef HAVE_LIBWRAP
-#ifndef HAVE_LIBWRAP_PROTOTYPES
-extern "C" {
 #include <tcpd.h>
-}
-#else
-#include <tcpd.h>
-#endif
 #include <syslog.h>
 #endif
 
@@ -57,6 +43,94 @@ ulong Mysqld_socket_listener::connection_errors_select= 0;
 ulong Mysqld_socket_listener::connection_errors_accept= 0;
 ulong Mysqld_socket_listener::connection_errors_tcpwrap= 0;
 
+#ifdef HAVE_PSI_STATEMENT_INTERFACE
+PSI_statement_info stmt_info_new_packet;
+#endif
+
+void net_before_header_psi(struct st_net *net, void *user_data, size_t /* unused: count */)
+{
+  THD *thd;
+  thd= static_cast<THD*> (user_data);
+  DBUG_ASSERT(thd != NULL);
+
+  if (thd->m_server_idle)
+  {
+    /*
+      The server is IDLE, waiting for the next command.
+      Technically, it is a wait on a socket, which may take a long time,
+      because the call is blocking.
+      Disable the socket instrumentation, to avoid recording a SOCKET event.
+      Instead, start explicitly an IDLE event.
+    */
+    MYSQL_SOCKET_SET_STATE(net->vio->mysql_socket, PSI_SOCKET_STATE_IDLE);
+    MYSQL_START_IDLE_WAIT(thd->m_idle_psi, &thd->m_idle_state);
+  }
+}
+
+void net_after_header_psi(struct st_net *net, void *user_data, size_t /* unused: count */, my_bool rc)
+{
+  THD *thd;
+  thd= static_cast<THD*> (user_data);
+  DBUG_ASSERT(thd != NULL);
+
+  if (thd->m_server_idle)
+  {
+    /*
+      The server just got data for a network packet header,
+      from the network layer.
+      The IDLE event is now complete, since we now have a message to process.
+      We need to:
+      - start a new STATEMENT event
+      - start a new STAGE event, within this statement,
+      - start recording SOCKET WAITS events, within this stage.
+      The proper order is critical to get events numbered correctly,
+      and nested in the proper parent.
+    */
+    MYSQL_END_IDLE_WAIT(thd->m_idle_psi);
+
+    if (! rc)
+    {
+      DBUG_ASSERT(thd->m_statement_psi == NULL);
+      thd->m_statement_psi= MYSQL_START_STATEMENT(&thd->m_statement_state,
+                                                  stmt_info_new_packet.m_key,
+                                                  thd->db().str,
+                                                  thd->db().length,
+                                                  thd->charset(), NULL);
+
+      THD_STAGE_INFO(thd, stage_starting);
+    }
+
+    /*
+      TODO: consider recording a SOCKET event for the bytes just read,
+      by also passing count here.
+    */
+    MYSQL_SOCKET_SET_STATE(net->vio->mysql_socket, PSI_SOCKET_STATE_ACTIVE);
+    thd->m_server_idle= false;
+  }
+}
+
+
+static void init_net_server_extension(THD *thd)
+{
+#ifdef HAVE_PSI_INTERFACE
+  /* Start with a clean state for connection events. */
+  thd->m_idle_psi= NULL;
+  thd->m_statement_psi= NULL;
+  thd->m_server_idle= false;
+  /* Hook up the NET_SERVER callback in the net layer. */
+  thd->m_net_server_extension.m_user_data= thd;
+  thd->m_net_server_extension.m_before_header= net_before_header_psi;
+  thd->m_net_server_extension.m_after_header= net_after_header_psi;
+
+  /* Activate this private extension for the mysqld server. */
+  thd->get_protocol_classic()->get_net()->extension=
+    &thd->m_net_server_extension;
+#else
+  thd->get_protocol_classic()->get_net()->extension= NULL;
+#endif
+}
+
+
 ///////////////////////////////////////////////////////////////////////////
 // Channel_info_local_socket implementation
 ///////////////////////////////////////////////////////////////////////////
@@ -877,22 +951,15 @@ Channel_info* Mysqld_socket_listener::listen_for_connection_event()
       syslog(LOG_AUTH | m_deny_severity,
              "refused connect from %s", eval_client(&req));
 
-#ifdef HAVE_LIBWRAP_PROTOTYPES
-      // Some distros have patched tcpd.h to have proper prototypes
       if (req.sink)
         (req.sink)(req.fd);
-#else
-      // Some distros have not patched tcpd.h
-      if (req.sink)
-        ((void (*)(int))req.sink)(req.fd);
-#endif
+
+      mysql_socket_shutdown(listen_sock, SHUT_RDWR);
+      mysql_socket_close(listen_sock);
       /*
         The connection was refused by TCP wrappers.
         There are no details (by client IP) available to update the host_cache.
       */
-      mysql_socket_shutdown(connect_sock, SHUT_RDWR);
-      mysql_socket_close(connect_sock);
-
       connection_errors_tcpwrap++;
       return NULL;
     }
