@@ -1,21 +1,14 @@
 /*
-   Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -306,7 +299,7 @@ ndb_serialize_cond(const Item *item, void *arg)
                 type != MYSQL_TYPE_GEOMETRY)
             {
               const NDBCOL *col= tab->getColumn(field->field_name);
-              assert(col);
+              DBUG_ASSERT(col);
               curr_cond->ndb_item= new Ndb_item(field, col->getColumnNo());
               context->dont_expect(Item::FIELD_ITEM);
               context->expect_no_field_result();
@@ -778,7 +771,7 @@ ndb_serialize_cond(const Item *item, void *arg)
           if (context->expecting(Item::STRING_ITEM) &&
               context->expecting_length(item->max_length)) 
           {
-#ifndef NDEBUG
+#ifndef DBUG_OFF
             char buff[256];
             String str(buff, 0, system_charset_info);
             const_cast<Item*>(item)->print(&str, QT_ORDINARY);
@@ -1039,7 +1032,7 @@ ndb_serialize_cond(const Item *item, void *arg)
             if (context->expecting(Item::STRING_ITEM) &&
                 context->expecting_length(item->max_length)) 
             {
-  #ifndef NDEBUG
+  #ifndef DBUG_OFF
               char buff[256];
               String str(buff, 0, system_charset_info);
               const_cast<Item*>(item)->print(&str, QT_ORDINARY);
@@ -1229,7 +1222,7 @@ ha_ndbcluster_cond::build_scan_filter_predicate(Ndb_cond * &cond,
       break;
     default:
       field= NULL; //Keep compiler happy
-      assert(0);
+      DBUG_ASSERT(0);
       break;
     }
     switch ((negated) ? 
@@ -1654,7 +1647,7 @@ int ha_ndbcluster_cond::generate_scan_filter_from_key(NdbInterpretedCode* code,
 {
   DBUG_ENTER("generate_scan_filter_from_key");
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   {
     DBUG_PRINT("info", ("key parts:%u length:%u",
                         key_info->user_defined_key_parts, key_info->key_length));
@@ -1686,7 +1679,7 @@ int ha_ndbcluster_cond::generate_scan_filter_from_key(NdbInterpretedCode* code,
               key_range has no count of parts so must test byte length.
               But this is not the place for following assert.
             */
-            // assert(ptr - key->key == key->length);
+            // DBUG_ASSERT(ptr - key->key == key->length);
             break;
           }
           key_part++;
@@ -1790,7 +1783,7 @@ int ha_ndbcluster_cond::generate_scan_filter_from_key(NdbInterpretedCode* code,
 
     DBUG_PRINT("info", ("Unknown hash index scan"));
     // enable to catch new cases when optimizer changes
-    // assert(false);
+    // DBUG_ASSERT(false);
   }
   while (0);
 
