@@ -717,8 +717,7 @@ static void set_psi(THD *thd)
 
 
 /**
-  Initializes physical thread to use with session service. The used
-  PSI key is key_thread_daemon_plugin
+  Initializes physical thread to use with session service.
 
   @param plugin Pointer to the plugin structure, passed to the plugin over
                 the plugin init function.
@@ -738,14 +737,6 @@ bool Srv_session::init_thread(const void *plugin)
     return true;
   }
 
-#ifdef HAVE_PSI_THREAD_INTERFACE
-  PSI_thread *psi= PSI_THREAD_CALL
-         (new_thread)(key_thread_daemon_plugin,
-                      NULL /*identity */,
-                      0 /* thread_id*/);
-  PSI_THREAD_CALL(set_thread)(psi);
-#endif
-
   server_session_threads.add(my_thread_self(), plugin);
 
   return false;
@@ -814,9 +805,6 @@ void Srv_session::deinit_thread()
   my_set_thread_local(THR_srv_session_thread, NULL);
 
   DBUG_ASSERT(my_get_thread_local(THR_stack_start_address));
-#ifdef HAVE_PSI_THREAD_INTERFACE
-  PSI_THREAD_CALL(delete_current_thread)();
-#endif
   my_set_thread_local(THR_stack_start_address, NULL);
   my_thread_end();
 }
