@@ -1,22 +1,14 @@
 /*****************************************************************************
 
-Copyright (c) 2013, 2023, Oracle and/or its affiliates.
+Copyright (c) 2013, 2016, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License, version 2.0,
-as published by the Free Software Foundation.
+This program is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free Software
+Foundation; version 2 of the License.
 
-This program is also distributed with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have included with MySQL.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License, version 2.0, for more details.
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -93,7 +85,7 @@ Datafile::open_or_create(bool read_only_mode)
 {
 	bool success;
 	ut_a(m_filepath != NULL);
-	ut_ad(m_handle.m_file == OS_FILE_CLOSED);
+	ut_ad(m_handle == OS_FILE_CLOSED);
 
 	m_handle = os_file_create(
 		innodb_data_file_key, m_filepath, m_open_flags,
@@ -116,7 +108,7 @@ dberr_t
 Datafile::open_read_only(bool strict)
 {
 	bool	success = false;
-	ut_ad(m_handle.m_file == OS_FILE_CLOSED);
+	ut_ad(m_handle == OS_FILE_CLOSED);
 
 	/* This function can be called for file objects that do not need
 	to be opened, which is the case when the m_filepath is NULL */
@@ -153,7 +145,7 @@ dberr_t
 Datafile::open_read_write(bool read_only_mode)
 {
 	bool	success = false;
-	ut_ad(m_handle.m_file == OS_FILE_CLOSED);
+	ut_ad(m_handle == OS_FILE_CLOSED);
 
 	/* This function can be called for file objects that do not need
 	to be opened, which is the case when the m_filepath is NULL */
@@ -185,9 +177,9 @@ void
 Datafile::init_file_info()
 {
 #ifdef _WIN32
-	GetFileInformationByHandle(m_handle.m_file, &m_file_info);
+	GetFileInformationByHandle(m_handle, &m_file_info);
 #else
-	fstat(m_handle.m_file, &m_file_info);
+	fstat(m_handle, &m_file_info);
 #endif	/* WIN32 */
 }
 
@@ -196,11 +188,13 @@ Datafile::init_file_info()
 dberr_t
 Datafile::close()
 {
-	 if (m_handle.m_file != OS_FILE_CLOSED) {
-		ibool   success = os_file_close(m_handle);
+	if (m_handle != OS_FILE_CLOSED) {
+		ibool	success = os_file_close(m_handle);
 		ut_a(success);
-		m_handle.m_file = OS_FILE_CLOSED;
+
+		m_handle = OS_FILE_CLOSED;
 	}
+
 	return(DB_SUCCESS);
 }
 
@@ -315,8 +309,10 @@ datafile.  The Datafile must already be open.
 dberr_t
 Datafile::read_first_page(bool read_only_mode)
 {
-	if (m_handle.m_file == OS_FILE_CLOSED) {
+	if (m_handle == OS_FILE_CLOSED) {
+
 		dberr_t err = open_or_create(read_only_mode);
+
 		if (err != DB_SUCCESS) {
 			return(err);
 		}
@@ -655,9 +651,10 @@ Datafile::validate_first_page(lsn_t*	flush_lsn,
 			ib::error()
 				<< "Encryption information in"
 				<< " datafile: " << m_filepath
-				<< " can't be decrypted,"
-				<< " please check if a keyring plugin"
-				<< " is loaded and initialized successfully.";
+				<< " can't be decrypted"
+				<< " , please confirm the keyfile"
+				<< " is match and keyring plugin"
+				<< " is loaded.";
 
 			m_is_valid = false;
 			free_first_page();
@@ -719,7 +716,7 @@ Datafile::find_space_id()
 {
 	os_offset_t	file_size;
 
-	ut_ad(m_handle.m_file != OS_FILE_CLOSED);
+	ut_ad(m_handle != OS_FILE_CLOSED);
 
 	file_size = os_file_get_size(m_handle);
 
