@@ -485,6 +485,7 @@ LatchDebug::LatchDebug()
 	LEVEL_MAP_INSERT(SYNC_RECV);
 	LEVEL_MAP_INSERT(SYNC_LOG_FLUSH_ORDER);
 	LEVEL_MAP_INSERT(SYNC_LOG);
+	LEVEL_MAP_INSERT(SYNC_LOG_WRITE);
 	LEVEL_MAP_INSERT(SYNC_PAGE_CLEANER);
 	LEVEL_MAP_INSERT(SYNC_PURGE_QUEUE);
 	LEVEL_MAP_INSERT(SYNC_TRX_SYS_HEADER);
@@ -766,6 +767,7 @@ LatchDebug::check_order(
 	case SYNC_FTS_CACHE_INIT:
 	case SYNC_PAGE_CLEANER:
 	case SYNC_LOG:
+	case SYNC_LOG_WRITE:
 	case SYNC_LOG_FLUSH_ORDER:
 	case SYNC_FILE_FORMAT_TAG:
 	case SYNC_DOUBLEWRITE:
@@ -1389,6 +1391,8 @@ sync_latch_meta_init()
 
 	LATCH_ADD(LOG_SYS, SYNC_LOG, log_sys_mutex_key);
 
+	LATCH_ADD(LOG_WRITE, SYNC_LOG_WRITE, log_sys_write_mutex_key);
+
 	LATCH_ADD(LOG_FLUSH_ORDER, SYNC_LOG_FLUSH_ORDER,
 		  log_flush_order_mutex_key);
 
