@@ -1,22 +1,14 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2023, Oracle and/or its affiliates.
+Copyright (c) 1996, 2016, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License, version 2.0,
-as published by the Free Software Foundation.
+This program is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free Software
+Foundation; version 2 of the License.
 
-This program is also distributed with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have included with MySQL.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License, version 2.0, for more details.
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -167,8 +159,8 @@ TrxUndoRsegsIterator::set_next()
 	/* We assume in purge of externally stored fields that
 	space id is in the range of UNDO tablespace space ids
 	unless space is system tablespace */
-	ut_a(srv_is_undo_tablespace(m_purge_sys->rseg->space)
-	     || is_system_tablespace(
+	ut_a(m_purge_sys->rseg->space <= srv_undo_tablespaces_open
+		|| is_system_tablespace(
 			m_purge_sys->rseg->space));
 
 	const page_size_t	page_size(m_purge_sys->rseg->page_size);
@@ -259,7 +251,7 @@ trx_purge_sys_create(
 	here only because the query threads code requires it. It is otherwise
 	quite unnecessary. We should get rid of it eventually. */
 	purge_sys->trx->id = 0;
-	purge_sys->trx->start_time = ut_time_monotonic();
+	purge_sys->trx->start_time = ut_time();
 	purge_sys->trx->state = TRX_STATE_ACTIVE;
 	purge_sys->trx->op_info = "purge trx";
 
@@ -382,10 +374,7 @@ trx_purge_add_update_undo_to_history(
 	if (update_rseg_history_len) {
 		os_atomic_increment_ulint(
 			&trx_sys->rseg_history_len, n_added_logs);
-		if (trx_sys->rseg_history_len
-		    > srv_n_purge_threads * srv_purge_batch_size) {
-			srv_wake_purge_thread_if_not_active();
-		}
+		srv_wake_purge_thread_if_not_active();
 	}
 
 	/* Write the trx number to the undo log header */
@@ -406,34 +395,19 @@ trx_purge_add_update_undo_to_history(
 	}
 }
 
-/** Remove undo log header from the history list.
-@param[in,out]	rseg_hdr	rollback segment header
-@param[in]	log_hdr		undo log segment header
-@param[in,out]	mtr		mini transaction. */
-static
-void
-trx_purge_remove_log_hdr(
-	trx_rsegf_t*	rseg_hdr,
-	trx_ulogf_t*	log_hdr,
-	mtr_t*		mtr)
-{
-	flst_remove(rseg_hdr + TRX_RSEG_HISTORY,
-		    log_hdr + TRX_UNDO_HISTORY_NODE, mtr);
-
-	os_atomic_decrement_ulint(&trx_sys->rseg_history_len, 1);
-}
-
-/** Frees an undo log segment which is in the history list. Removes the
-undo log hdr from the history list.
-@param[in,out]	rseg		rollback segment
-@param[in]	hdr_addr	file address of log_hdr
-@param[in]	noredo		skip redo logging. */
+/**********************************************************************//**
+Frees an undo log segment which is in the history list. Cuts the end of the
+history list at the youngest undo log in this segment. */
 static
 void
 trx_purge_free_segment(
-	trx_rseg_t*	rseg,
-	fil_addr_t	hdr_addr,
-	bool		noredo)
+/*===================*/
+	trx_rseg_t*	rseg,		/*!< in: rollback segment */
+	fil_addr_t	hdr_addr,	/*!< in: the file address of log_hdr */
+	ulint		n_removed_logs,	/*!< in: count of how many undo logs we
+					will cut off from the end of the
+					history list */
+	bool		noredo)		/*!< in: skip redo logging */
 {
 	mtr_t		mtr;
 	trx_rsegf_t*	rseg_hdr;
@@ -499,7 +473,10 @@ trx_purge_free_segment(
 	history list: otherwise, in case of a database crash, the segment
 	could become inaccessible garbage in the file space. */
 
-	trx_purge_remove_log_hdr(rseg_hdr, log_hdr, &mtr);
+	flst_cut_end(rseg_hdr + TRX_RSEG_HISTORY,
+		     log_hdr + TRX_UNDO_HISTORY_NODE, n_removed_logs, &mtr);
+
+	os_atomic_decrement_ulint(&trx_sys->rseg_history_len, n_removed_logs);
 
 	do {
 
@@ -541,6 +518,7 @@ trx_purge_truncate_rseg_history(
 	page_t*		undo_page;
 	trx_ulogf_t*	log_hdr;
 	trx_usegf_t*	seg_hdr;
+	ulint		n_removed_logs	= 0;
 	mtr_t		mtr;
 	trx_id_t	undo_trx_no;
 	const bool	noredo		= trx_sys_is_noredo_rseg_slot(
@@ -587,6 +565,13 @@ loop:
 				hdr_addr.boffset, limit->undo_no);
 		}
 
+		os_atomic_decrement_ulint(
+			&trx_sys->rseg_history_len, n_removed_logs);
+
+		flst_truncate_end(rseg_hdr + TRX_RSEG_HISTORY,
+				  log_hdr + TRX_UNDO_HISTORY_NODE,
+				  n_removed_logs, &mtr);
+
 		mutex_exit(&(rseg->mutex));
 		mtr_commit(&mtr);
 
@@ -595,6 +580,7 @@ loop:
 
 	prev_hdr_addr = trx_purge_get_log_from_hist(
 		flst_get_prev_addr(log_hdr + TRX_UNDO_HISTORY_NODE, &mtr));
+	n_removed_logs++;
 
 	seg_hdr = undo_page + TRX_UNDO_SEG_HDR;
 
@@ -606,14 +592,10 @@ loop:
 		mutex_exit(&(rseg->mutex));
 		mtr_commit(&mtr);
 
-		/* calls the trx_purge_remove_log_hdr()
-		inside trx_purge_free_segment(). */
-		trx_purge_free_segment(rseg, hdr_addr, noredo);
+		trx_purge_free_segment(rseg, hdr_addr, n_removed_logs, noredo);
 
+		n_removed_logs = 0;
 	} else {
-		/* Remove the log hdr from the rseg history. */
-		trx_purge_remove_log_hdr(rseg_hdr, log_hdr, &mtr);
-
 		mutex_exit(&(rseg->mutex));
 		mtr_commit(&mtr);
 	}
@@ -695,7 +677,7 @@ namespace undo {
 		/* Step-2: Create the log file, open it and write 0 to
 		indicate init phase. */
 		bool            ret;
-		pfs_os_file_t	handle = os_file_create(
+		os_file_t	handle = os_file_create(
 			innodb_log_file_key, log_file_name, OS_FILE_CREATE,
 			OS_FILE_NORMAL, OS_LOG_FILE, srv_read_only_mode, &ret);
 		if (!ret) {
@@ -753,7 +735,7 @@ namespace undo {
 		/* Step-2: Open log file and write magic number to
 		indicate done phase. */
 		bool    ret;
-		pfs_os_file_t	handle =
+		os_file_t	handle =
 			os_file_create_simple_no_error_handling(
 				innodb_log_file_key, log_file_name,
 				OS_FILE_OPEN, OS_FILE_READ_WRITE,
@@ -823,7 +805,7 @@ namespace undo {
 
 		if (exist) {
 			bool    ret;
-			pfs_os_file_t	handle =
+			os_file_t	handle =
 				os_file_create_simple_no_error_handling(
 					innodb_log_file_key, log_file_name,
 					OS_FILE_OPEN, OS_FILE_READ_WRITE,
@@ -916,7 +898,7 @@ trx_purge_mark_undo_for_truncate(
 	b. At-least 2 UNDO redo rseg/undo logs (besides the default rseg-0)
 	b. At-least 1 UNDO tablespace size > threshold. */
 	if (srv_undo_tablespaces_active < 2
-	    || (srv_rollback_segments < (1 + srv_tmp_undo_logs + 2))) {
+	    || (srv_undo_logs < (1 + srv_tmp_undo_logs + 2))) {
 		return;
 	}
 
@@ -926,8 +908,6 @@ trx_purge_mark_undo_for_truncate(
 
 	for (ulint i = 1; i <= srv_undo_tablespaces_active; i++) {
 
-		ut_ad(srv_undo_space_id_start != 0);
-
 		if (fil_space_get_size(space_id)
 		    > (srv_max_undo_log_size / srv_page_size)) {
 			/* Tablespace qualifies for truncate. */
@@ -936,21 +916,13 @@ trx_purge_mark_undo_for_truncate(
 			break;
 		}
 
-		space_id++;
-
-		if (space_id >= (srv_undo_space_id_start
-				 + srv_undo_tablespaces_active)) {
+		space_id = ((space_id + 1) % (srv_undo_tablespaces_active + 1));
+		if (space_id == 0) {
 			/* Note: UNDO tablespace ids starts from 1. */
-			space_id = srv_undo_space_id_start;
-		}
-
-		if (undo_trunc->is_marked()) {
-			break;
+			++space_id;
 		}
 	}
 
-	undo_trunc->set_scan_start(space_id);
-
 	/* Couldn't make any selection. */
 	if (!undo_trunc->is_marked()) {
 		return;
@@ -1306,7 +1278,6 @@ trx_purge_rseg_get_next_history_log(
 		mutex_exit(&(rseg->mutex));
 		mtr_commit(&mtr);
 
-#ifdef UNIV_DEBUG
 		trx_sys_mutex_enter();
 
 		/* Add debug code to track history list corruption reported
@@ -1320,19 +1291,14 @@ trx_purge_rseg_get_next_history_log(
 		if (trx_sys->rseg_history_len > 2000000) {
 			ib::warn() << "Purge reached the head of the history"
 				" list, but its length is still reported as "
-				<< trx_sys->rseg_history_len << " which is"
-				" unusually high.";
-			ib::info() << "This can happen for multiple reasons";
-			ib::info() << "1. A long running transaction is"
-				" withholding purging of undo logs or a read"
-				" view is open. Please try to commit the long"
-				" running transaction.";
-			ib::info() << "2. Try increasing the number of purge"
-				" threads to expedite purging of undo logs.";
+				<< trx_sys->rseg_history_len << "! Make"
+				" a detailed bug report, and submit it to"
+				" http://bugs.mysql.com";
+			ut_ad(0);
 		}
 
 		trx_sys_mutex_exit();
-#endif
+
 		return;
 	}
 
@@ -1562,6 +1528,7 @@ trx_purge_get_next_rec(
 		undo_page = trx_undo_page_get_s_latched(
 			page_id_t(space, page_no), page_size, &mtr);
 
+		rec = undo_page + offset;
 	} else {
 		page = page_align(rec2);
 
@@ -1576,8 +1543,10 @@ trx_purge_get_next_rec(
 		}
 	}
 
-	rec_copy = trx_undo_rec_copy(undo_page, offset, heap);
+	rec_copy = trx_undo_rec_copy(rec, heap);
+
 	mtr_commit(&mtr);
+
 	return(rec_copy);
 }
 
@@ -1749,9 +1718,7 @@ trx_purge_dml_delay(void)
 	Note: we do a dirty read of the trx_sys_t data structure here,
 	without holding trx_sys->mutex. */
 
-	if (srv_max_purge_lag > 0
-	    && trx_sys->rseg_history_len
-	       > srv_n_purge_threads * srv_purge_batch_size) {
+	if (srv_max_purge_lag > 0) {
 		float	ratio;
 
 		ratio = float(trx_sys->rseg_history_len) / srv_max_purge_lag;
