@@ -1,21 +1,13 @@
-/* Copyright (C) 2008 MySQL AB
-   Use is subject to license terms
+/* Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -26,9 +18,13 @@
 
 #include "SignalData.hpp"
 
+#define JAM_FILE_ID 40
+
+
 class EnableComReq  {
   friend class Qmgr;
-  friend class Cmvmi;
+  friend class Trpman;
+  friend class TrpmanProxy;
 
 public:
   STATIC_CONST( SignalLength = 2 + NodeBitmask::Size );
@@ -41,6 +37,8 @@ private:
 
 class EnableComConf  {
   friend class Qmgr;
+  friend class Trpman;
+  friend class TrpmanProxy;
   friend class Cmvmi;
 
 public:
@@ -52,4 +50,7 @@ private:
   Uint32 m_nodeIds[NodeBitmask::Size];
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
