@@ -1,21 +1,14 @@
 /*
-   Copyright (c) 2004, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2004, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -98,6 +91,51 @@ const char *Ndb_cluster_connection::get_connected_host() const
   return 0;
 }
 
+int
+Ndb_cluster_connection::unset_recv_thread_cpu(Uint32 recv_thread_id)
+{
+  if (m_impl.m_transporter_facade)
+  {
+    return m_impl.m_transporter_facade->unset_recv_thread_cpu(recv_thread_id);
+  }
+  return -1;
+}
+
+int
+Ndb_cluster_connection::set_recv_thread_cpu(Uint16 *cpuid_array,
+                                            Uint32 array_len,
+                                            Uint32 recv_thread_id)
+{
+  if (m_impl.m_transporter_facade)
+  {
+    return m_impl.m_transporter_facade->set_recv_thread_cpu(cpuid_array,
+                                                            array_len,
+                                                            recv_thread_id);
+  }
+  return -1;
+}
+
+int
+Ndb_cluster_connection::set_recv_thread_activation_threshold(Uint32 threshold)
+{
+  TransporterFacade *fac = m_impl.m_transporter_facade;
+  if (fac)
+  {
+    return fac->set_recv_thread_activation_threshold(threshold);
+  }
+  return -1;
+}
+
+int
+Ndb_cluster_connection::get_recv_thread_activation_threshold() const
+{
+  if (m_impl.m_transporter_facade)
+  {
+    return m_impl.m_transporter_facade->get_recv_thread_activation_threshold();
+  }
+  return -1;
+}
+
 const char *Ndb_cluster_connection::get_connectstring(char *buf,
 						      int buf_sz) const
 {
@@ -211,13 +249,13 @@ Ndb_cluster_connection_impl::get_next_alive_node(Ndb_cluster_connection_node_ite
 
   while ((id = get_next_node(iter)))
   {
-    tp->lock_mutex();
+    tp->lock_poll_mutex();
     if (tp->get_node_alive(id) != 0)
     {
-      tp->unlock_mutex();
+      tp->unlock_poll_mutex();
       return id;
     }
-    tp->unlock_mutex();
+    tp->unlock_poll_mutex();
   }
   return 0;
 }
@@ -242,7 +280,7 @@ Ndb_cluster_connection::max_nodegroup()
     return 0;
 
   Bitmask<MAX_NDB_NODES> ng;
-  tp->lock_mutex();
+  tp->lock_poll_mutex();
   for(unsigned i= 0; i < no_db_nodes(); i++)
   {
     //************************************************
@@ -252,7 +290,7 @@ Ndb_cluster_connection::max_nodegroup()
     if (n.is_confirmed() && n.m_state.nodeGroup <= MAX_NDB_NODES)
       ng.set(n.m_state.nodeGroup);
   }
-  tp->unlock_mutex();
+  tp->unlock_poll_mutex();
 
   if (ng.isclear())
     return 0;
@@ -274,7 +312,7 @@ int Ndb_cluster_connection::get_no_ready()
     return -1;
 
   unsigned int foundAliveNode = 0;
-  tp->lock_mutex();
+  tp->lock_poll_mutex();
   for(unsigned i= 0; i < no_db_nodes(); i++)
   {
     //************************************************
@@ -284,7 +322,7 @@ int Ndb_cluster_connection::get_no_ready()
       foundAliveNode++;
     }
   }
-  tp->unlock_mutex();
+  tp->unlock_poll_mutex();
 
   return foundAliveNode;
 }
@@ -307,13 +345,13 @@ Ndb_cluster_connection::wait_until_ready(int timeout,
   int milliCounter = 0;
   int noChecksSinceFirstAliveFound = 0;
   do {
-    unsigned int foundAliveNode = get_no_ready();
+    const Uint32 unconnected_nodes = m_impl.get_unconnected_nodes();
 
-    if (foundAliveNode == no_db_nodes())
+    if (unconnected_nodes == 0)
     {
       DBUG_RETURN(0);
     }
-    else if (foundAliveNode > 0)
+    else if (unconnected_nodes < no_db_nodes())
     {
       noChecksSinceFirstAliveFound++;
       // 100 ms delay -> 10*
@@ -369,7 +407,8 @@ Ndb_cluster_connection_impl(const char * connect_string,
     m_first_ndb_object(0),
     m_latest_error_msg(),
     m_latest_error(0),
-    m_max_trans_id(0)
+    m_max_trans_id(0),
+    m_multi_wait_group(0)
 {
   DBUG_ENTER("Ndb_cluster_connection");
   DBUG_PRINT("enter",("Ndb_cluster_connection this=0x%lx", (long) this));
@@ -387,11 +426,17 @@ Ndb_cluster_connection_impl(const char * connect_string,
       of order.  Same applies for regular ndbapi user.
     */
     g_eventLogger->setRepeatFrequency(0);
+
+#ifdef VM_TRACE
+    ndb_print_state_mutex= NdbMutex_Create();
+#endif
+
   }
   NdbMutex_Unlock(g_ndb_connection_mutex);
 
   m_event_add_drop_mutex= NdbMutex_Create();
   m_new_delete_ndb_mutex = NdbMutex_Create();
+  m_new_delete_ndb_cond = NdbCondition_Create();
 
   m_connect_thread= 0;
   m_connect_callback= 0;
@@ -399,10 +444,6 @@ Ndb_cluster_connection_impl(const char * connect_string,
   /* Clear global stats baseline */
   memset(globalApiStatsBaseline, 0, sizeof(globalApiStatsBaseline));
 
-#ifdef VM_TRACE
-  if (ndb_print_state_mutex == NULL)
-    ndb_print_state_mutex= NdbMutex_Create();
-#endif
   m_config_retriever=
     new ConfigRetriever(connect_string, force_api_nodeid,
                         NDB_VERSION, NDB_MGM_NODE_TYPE_API);
@@ -439,20 +480,23 @@ Ndb_cluster_connection_impl::~Ndb_cluster_connection_impl()
 {
   DBUG_ENTER("~Ndb_cluster_connection");
 
-  if (m_first_ndb_object != 0)
+  // Wait until all Ndb instances belonging to this Ndb_cluster_connection
+  // have been released(they have references to the TransporterFacade)
+  NdbMutex_Lock(m_new_delete_ndb_mutex);
+  if (m_first_ndb_object)
   {
-    g_eventLogger->warning("Deleting Ndb_cluster_connection with Ndb-object"
-                           " not deleted");
-    Ndb * p = m_first_ndb_object;
-    printf("this: %p Ndb-object(s): ", (Ndb_cluster_connection*)this);
-    while (p)
+    g_eventLogger->warning("Waiting for Ndb instances belonging to "
+                           "Ndb_cluster_connection %p to be deleted...",
+                           this);
+
+    while(m_first_ndb_object)
     {
-      printf("%p ", p);
-      p = p->theImpl->m_next_ndb_object;
+      NdbCondition_WaitTimeout(m_new_delete_ndb_cond,
+                               m_new_delete_ndb_mutex,
+                               1000);
     }
-    printf("\n");
-    fflush(stdout);
   }
+  NdbMutex_Unlock(m_new_delete_ndb_mutex);
 
   if (m_transporter_facade != 0)
   {
@@ -480,18 +524,17 @@ Ndb_cluster_connection_impl::~Ndb_cluster_connection_impl()
     delete m_config_retriever;
     m_config_retriever= NULL;
   }
-#ifdef VM_TRACE
-  if (ndb_print_state_mutex != NULL)
-  {
-    NdbMutex_Destroy(ndb_print_state_mutex);
-    ndb_print_state_mutex= NULL;
-  }
-#endif
 
   NdbMutex_Lock(g_ndb_connection_mutex);
   if(--g_ndb_connection_count == 0)
   {
     NdbColumnImpl::destory_pseudo_columns();
+
+#ifdef VM_TRACE
+    NdbMutex_Destroy(ndb_print_state_mutex);
+    ndb_print_state_mutex= NULL;
+#endif
+
   }
   NdbMutex_Unlock(g_ndb_connection_mutex);
 
@@ -502,7 +545,15 @@ Ndb_cluster_connection_impl::~Ndb_cluster_connection_impl()
   if (m_new_delete_ndb_mutex)
     NdbMutex_Destroy(m_new_delete_ndb_mutex);
   m_new_delete_ndb_mutex = 0;
+
+  if (m_new_delete_ndb_cond)
+    NdbCondition_Destroy(m_new_delete_ndb_cond);
+  m_new_delete_ndb_cond = 0;
   
+  if(m_multi_wait_group)
+    delete m_multi_wait_group;
+  m_multi_wait_group = 0;
+
   DBUG_VOID_RETURN;
 }
 
@@ -540,6 +591,10 @@ Ndb_cluster_connection_impl::link_ndb_object(Ndb* p)
   m_first_ndb_object = p;
   
   p->theFirstTransId += m_max_trans_id;
+
+  // Wake up anyone waiting for changes to the Ndb instance list
+  NdbCondition_Broadcast(m_new_delete_ndb_cond);
+
   unlock_ndb_objects();
 }
 
@@ -583,6 +638,9 @@ Ndb_cluster_connection_impl::unlink_ndb_object(Ndb* p)
     globalApiStatsBaseline[i] += p->theImpl->clientStats[i];
   }
 
+  // Wake up anyone waiting for changes to the Ndb instance list
+  NdbCondition_Broadcast(m_new_delete_ndb_cond);
+
   unlock_ndb_objects();  
 }
 
@@ -703,6 +761,45 @@ Ndb_cluster_connection_impl::get_db_nodes(Uint8 arr[MAX_NDB_NODES]) const
   return cnt;
 }
 
+Uint32
+Ndb_cluster_connection_impl::get_unconnected_nodes() const
+{
+  TransporterFacade *tp = m_transporter_facade;
+
+  NodeBitmask db_nodes;  // All data nodes known by configuration
+  NodeBitmask connected; // All nodes connected
+  NodeBitmask started;   // All started nodes known by connected db nodes
+
+  tp->lock_poll_mutex();
+  for(unsigned i= 0; i < m_all_nodes.size(); i++)
+  {
+    const Uint32 node_id = m_all_nodes[i].id;
+    db_nodes.set(node_id);
+    const trp_node& node = tp->theClusterMgr->getNodeInfo(node_id);
+    if (!node.m_alive)
+    {
+      continue;
+    }
+    connected.set(node_id);
+    started.bitOR(node.m_state.m_connected_nodes);
+  }
+  tp->unlock_poll_mutex();
+
+  if (connected.count() == 0)
+  {
+    /**
+     * No db nodes connected, means all unconnected.
+     */
+    return m_all_nodes.size();
+  }
+
+  /**
+   * Return count of started but not connected db nodes
+   */
+  started.bitAND(db_nodes);
+  return started.bitANDC(connected).count();
+}
+
 int
 Ndb_cluster_connection_impl::configure(Uint32 nodeId,
                                        const ndb_mgm_configuration &config)
@@ -727,24 +824,58 @@ Ndb_cluster_connection_impl::configure(Uint32 nodeId,
       m_config.m_batch_size= batch_size;
     }
 
-    // Configure timeouts
-    Uint32 timeout = 120000;
-    for (iter.first(); iter.valid(); iter.next())
-    {
-      Uint32 tmp1 = 0, tmp2 = 0;
-      iter.get(CFG_DB_TRANSACTION_CHECK_INTERVAL, &tmp1);
-      iter.get(CFG_DB_TRANSACTION_DEADLOCK_TIMEOUT, &tmp2);
-      tmp1 += tmp2;
-      if (tmp1 > timeout)
-        timeout = tmp1;
-    }
-    m_config.m_waitfor_timeout = timeout;
-
     Uint32 queue = 0;
     if (!iter.get(CFG_DEFAULT_OPERATION_REDO_PROBLEM_ACTION, &queue))
     {
       m_config.m_default_queue_option = queue;
     }
+
+    Uint32 default_hashmap_size = 0;
+    if (!iter.get(CFG_DEFAULT_HASHMAP_SIZE, &default_hashmap_size) &&
+        default_hashmap_size != 0)
+    {
+      m_config.m_default_hashmap_size = default_hashmap_size;
+    }
+    // If DefaultHashmapSize is not set or zero, use the minimum
+    // value set (not zero) for any other node, since this size
+    // should be supported by the other nodes.  Also this allows
+    // the DefaultHashmapSize to be set for the entire cluster
+    // if set for a single node or node type.
+    // Otherwise use NDB_DEFAULT_HASHMAP_BUCKETS
+    if (default_hashmap_size == 0)
+    {
+      // Use new iterator to leave iter valid.
+      ndb_mgm_configuration_iterator iterall(config, CFG_SECTION_NODE);
+      for (; iterall.valid(); iterall.next())
+      {
+        Uint32 tmp = 0;
+        if (!iterall.get(CFG_DEFAULT_HASHMAP_SIZE, &tmp) &&
+            tmp != 0 &&
+            ((default_hashmap_size == 0) || (tmp < default_hashmap_size)))
+          default_hashmap_size = tmp;
+      }
+      if (default_hashmap_size == 0)
+        default_hashmap_size = NDB_DEFAULT_HASHMAP_BUCKETS;
+
+      m_config.m_default_hashmap_size = default_hashmap_size;
+    }
+
+    // Configure timeouts
+    {
+      Uint32 timeout = 120000;
+      // Use new iterator to leave iter valid.
+      ndb_mgm_configuration_iterator iterall(config, CFG_SECTION_NODE);
+      for (; iterall.valid(); iterall.next())
+      {
+        Uint32 tmp1 = 0, tmp2 = 0;
+        iterall.get(CFG_DB_TRANSACTION_CHECK_INTERVAL, &tmp1);
+        iterall.get(CFG_DB_TRANSACTION_DEADLOCK_TIMEOUT, &tmp2);
+        tmp1 += tmp2;
+        if (tmp1 > timeout)
+          timeout = tmp1;
+      }
+      m_config.m_waitfor_timeout = timeout;
+    }
   }
   DBUG_RETURN(init_nodes_vector(nodeId, config));
 }
@@ -823,22 +954,38 @@ int Ndb_cluster_connection_impl::connect(int no_retries,
       DBUG_PRINT("exit", ("no m_config_retriever, ret: -1"));
       DBUG_RETURN(-1);
     }
-    if (m_config_retriever->do_connect(no_retries,
-                                       retry_delay_in_seconds,
-                                       verbose))
+
+    // the allocNodeId function will connect if not connected
+    int alloc_error;
+    Uint32 nodeId = m_config_retriever->allocNodeId(no_retries,
+                                                   retry_delay_in_seconds,
+                                                   verbose, alloc_error);
+    if (!nodeId)
     {
-      char buf[1024];
-      m_latest_error = 1;
-      m_latest_error_msg.assfmt("Connect using '%s' timed out",
-                                get_connectstring(buf, sizeof(buf)));
-      DBUG_PRINT("exit", ("mgmt server not up yet, ret: 1"));
-      DBUG_RETURN(1); // mgmt server not up yet
+      // Failed to allocate nodeid from mgmt server, find out
+      // the cause and set proper error message
+
+      if (!m_config_retriever->is_connected())
+      {
+        // Could not connect to mgmt server
+        m_latest_error = alloc_error;
+        m_latest_error_msg.assfmt("%s", m_config_retriever->getErrorString());
+        DBUG_RETURN(1); // Recoverable error
+      }
+
+      if (alloc_error == NDB_MGM_ALLOCID_ERROR)
+      {
+        // A nodeid for this node was found in config, but it was not
+        // free right now. Retry later and it might be free.
+        m_latest_error = alloc_error;
+        m_latest_error_msg.assfmt("%s", m_config_retriever->getErrorString());
+        DBUG_RETURN(1); // Recoverable error
+      }
+
+      // Fatal error, use default error
+      break;
     }
 
-    Uint32 nodeId = m_config_retriever->allocNodeId(4/*retries*/,
-                                                    3/*delay*/);
-    if(nodeId == 0)
-      break;
     ndb_mgm_configuration * props = m_config_retriever->getConfig(nodeId);
     if(props == 0)
       break;
@@ -987,5 +1134,117 @@ Ndb_cluster_connection::collect_client_stats(Uint64* statsArr, Uint32 sz)
   return relevant;
 }
 
+void
+Ndb_cluster_connection::set_max_adaptive_send_time(Uint32 milliseconds)
+{
+  m_impl.m_transporter_facade->setSendThreadInterval(milliseconds);
+}
+
+Uint32
+Ndb_cluster_connection::get_max_adaptive_send_time()
+{
+  return m_impl.m_transporter_facade->getSendThreadInterval();
+}
+
+NdbWaitGroup *
+Ndb_cluster_connection::create_ndb_wait_group(int size)
+{
+  if(m_impl.m_multi_wait_group == NULL)
+  {
+    m_impl.m_multi_wait_group = new NdbWaitGroup(this, size);
+    return m_impl.m_multi_wait_group;
+  }
+  else
+  {
+    return NULL;  // NdbWaitGroup already exists
+  }
+}
+
+bool
+Ndb_cluster_connection::release_ndb_wait_group(NdbWaitGroup *group)
+{
+  if(m_impl.m_multi_wait_group && m_impl.m_multi_wait_group == group)
+  {
+    delete m_impl.m_multi_wait_group;
+    m_impl.m_multi_wait_group = 0;
+    return true;
+  }
+  else
+  {
+    return false;
+  }
+}
+
 template class Vector<Ndb_cluster_connection_impl::Node>;
 
+int
+Ndb_cluster_connection::wait_until_ready(const int * nodes, int cnt,
+                                         int timeout)
+{
+  DBUG_ENTER("Ndb_cluster_connection::wait_until_ready(nodelist)");
+
+  NodeBitmask mask;
+  for (int i = 0; i < cnt; i++)
+  {
+    if (nodes[i] <= 0 || nodes[i] > (int)mask.max_size())
+    {
+      DBUG_RETURN(-1);
+    }
+    mask.set(nodes[i]);
+  }
+
+  TransporterFacade *tp = m_impl.m_transporter_facade;
+  if (tp == 0)
+  {
+    DBUG_RETURN(-1);
+  }
+  if (tp->ownId() == 0)
+  {
+    DBUG_RETURN(-1);
+  }
+
+  timeout *= 10; // try each 100ms
+
+  NodeBitmask dead;
+  NodeBitmask alive;
+  do
+  {
+    dead.clear();
+    alive.clear();
+    tp->lock_poll_mutex();
+    for(unsigned i= 0; i < no_db_nodes(); i++)
+    {
+      //************************************************
+      // If any node is answering, ndb is answering
+      //************************************************
+      if (tp->get_node_alive(m_impl.m_all_nodes[i].id) != 0)
+        alive.set(m_impl.m_all_nodes[i].id);
+      else
+        dead.set(m_impl.m_all_nodes[i].id);
+    }
+    tp->unlock_poll_mutex();
+
+    if (alive.contains(mask))
+    {
+      DBUG_RETURN(mask.count());
+    }
+
+    NodeBitmask all;
+    all.bitOR(alive);
+    all.bitOR(dead);
+    if (!all.contains(mask))
+    {
+      DBUG_RETURN(-1);
+    }
+
+    if (timeout == 0)
+      break;
+
+    timeout--;
+    NdbSleep_MilliSleep(100);
+  } while (true);
+
+  mask.bitAND(alive);
+  DBUG_RETURN(mask.count());
+}
+
